pro ejecta_b_vm, deg_per_bin, n_part, p_out, s_out, p_edge, dm, v_x, v_low=v_low, dbug=dbug
; 
; Calculate the surface deposits of ejecta using input parameters
; and a low-resolution shape model of Bennu (v28, 12-m facets, smoothed).
; The velocity and mass of ejecta are varied according to the equations in 
; Housen and Holsapple 2011 for gravity scaling. In thier work, x is the
; radial distance from the crater center and used as a parameter for velocity
; mass of ejecta. Here, all material is ejected at the crater edge, but x is
; still used as a parameter to link the velocity and mass of ejecta. 
; See "Ejecta field simulation.docx" for details. 
; 
; Assumes that tilt is zero (surface normals are aligned with radial vector)
; where particles are ejected. This appears correct for the terrain surrounding
; the EX07 crater at 44S, 325E. Cannot use surface tilts from shape model
; because reflect the shape of the crater. 
; 
; The terrain near the crater is smoothed to better represent the terrrain before
; the crater was formed. For example, the crater rim did not exist and was formed
; by the slowest-speed ejecta. 
; 
; Trajectory alculations are high fidelity and done in an inertial frame. Velocity
; due to Bennu's rotation is explicity added. When including higher velocities, 
; trajectory calculations are stopped after 3600 seconds (particles have escaped or are
; in orbit). 
; 
; Uses Rodrigues' rotation formula (see Wikipedia) to get the initial ejection vector
; based on the surface normal and the ejection angle.
; 
; All longitudes in -180 to +180 degrees or -pi to +pi radians.
; 
; Input 
;   deg_per_bin= size of bins for storing output data
;   n_part= number of particles distributed evenly around crater edge
;     Also used for ns, the number of velocity and mass increments, so that one number 
;     will set all resolution-related variables. 
;   v_low= Nominal velocity at crater edge. If not provided, calculated using gravity scaling. 
;   
; Output
;   p_out[np,nxadj,6]= impact data for each ejected particle: lon,lat,angle,v,v_surface,time
;     for each velocity from each location. 
;   s_out[lon,lat,4]= surface location of ejecta: mass per bin, mean surface speed, std of speeds, count
;   p_edge[np,2]= longitude and latitude of each point on crater edge
;   dm[*]= mass fraction for each speed.
;   v_x[*]= speeds of modeled ejecta
; 
; 
; Created October 2019, Perry
;   Modified from ejecta.pro, which had an ellipsoid surface instead of Bennu,
;     ejecta_b.pro, which used a single velocity for ejecta.
;   Modified January 2021, Perry: higher maximum velocity to better plot mass fractions.
;     Added max time before assumed to be ejected. 
;

; Set default: no debugging
if n_elements(dbug) eq 0 then dbug=0

; Set Bennu parameters
rot_period= 4.3*3600. ; Rotation period (s)
;rot_period= 8.0*3600. ; Rotation period (s) for slower rotation
theta_deg= 360./rot_period ; rotation rate degrees/s
omega= 2.*!pi/rot_period  ; rotation rate, radians/s
GM=  4.892  ; m3/s2

; Get shape model (DTM in FITS format)
setoladir, 'SPCDTM' ; Location of the DTM
dtmfn= 'g_12630mm_spc_dtm_0000n00000_v028.fits'  ; v28, 12-m facets
shapedtm= mrdfits(dtmfn, 0, header)

; Extract the radius and smooth to reduce effects of ejecta-deposit topography
sz_dtm= size(shapedtm)
smooth_factor= 15
dtmllr= smooth(shapedtm[*,*,2],smooth_factor, /edge_wrap) * 1000. ; meters

; Set crater parameters
r_crater= 35.  ;meters
cll= [-35.0,-45.0] ;Longitude,latitude of crater center
;cll= [0.0,0.0] ;Longitude,latitude of crater center if at equator
;cll= [325.0,-45.0] ;Longitude,latitude of crater center
;cll= [0.0,-45.0] ;Longitude,latitude of crater center for testing
cllr= cll/ !radeg

; Get xyz of crater center and radius of crater center from Bennu center
llsurf_to_xyz, cll, dtmllr, cr, cllxyz

; Set constants used to calculate ejecta speed and mass
mu= 0.4 ;
p=0.3
n2= 1.3
max_v= 0.08 ; Maximum velocity (higher velocities escape or drive far north)
;max_v= 0.3 ; Higher maximum velocity to see effect on ejecta mass fractions
if max_v gt 0.12 then print, '****Warning, high velocity (escaped) particles not plotted correctly'
t_max= 3600.  ;Maximum time before particle assumed to be ejected
angd= 45. ; Ejection angle relative to surface
vang= angd/!radeg ; Ejection angle in radians

; Calculate the nominal velocity at the crater edge if not provided
g_crater= GM/cr^2. - omega^2. * cr ; gravity at crater center
if n_elements(v_low) eq 0 then begin
  v_low= sqrt(r_crater * g_crater)  ; Gravity scaling
  print, 'Nominal v at edge (m/s):', v_low
endif

; Create x array, the distance of ejecta from crater center
nx= n_part  ;Number of velocity and mass increments (use number of particles to set resolution)
;nx= 100.  ; Number of velocity and mass increments
;nx= 400.  ; Number of velocity and mass increments (for high resolution plots)
x= findgen(nx+1)/nx * n2*r_crater  ;Only used as parameter for speed and mass

; Calculate the calibration coeficient for ejecta speed
v_x= (1.-x/n2/r_crater)^p/x^(1./mu)  ;Uncalibrated speed
iR= where(x ge r_crater)  ; Index for crater edge 
c_R= v_low/v_x[iR[0]] ; Want v_x = v_low at crater edge
v_x= c_R * v_x  ; Later truncated to used values, only

; Caluate the total mass
mtot= (x/r_crater)^3.
mtot= mtot/mtot[nx] ; Normalized to 1

; Calculate the delta mass for each speed
dm= fltarr(nx+1)
for im= 1, nx do dm[im]= mtot[im] - mtot[im-1]

; Find index for maximum velocity (speed)
imax_v= where(v_x ge max_v)
x_min= imax_v[-1] ; Lower indices have higher velocities

; Find index for minimum velocity (speed)
imin_v= where(v_x le 0.25* v_low) ; Lowest velocity is 25% of nominal v at crater edge
x_max= imin_v[0] ; Lower indices have higher velocities
nxadj= x_max-x_min+1 ; Number of indeces for speed and mass

; Redifine speed and mass to values of interest, only
v_temp= v_x[x_min:x_max]
v_x= v_temp
m_temp= dm[x_min:x_max]
dm= m_temp
print, 'Velocity range (m/s):', v_x[-1], v_x[0]

; Set time interval for propagating trajectory 
t_delta= 2.0  ; Seconds

; Set resolution parameter
np= n_part  ; Number of crater locations

; Create output arrays
p_out= fltarr(np, nxadj, 6)  ; Impact data for each particle
nbin= 360./deg_per_bin  ; Number of longitude bins
s_out= fltarr(nbin, nbin/2, 4) ;Array of surface locations: mass, surface speed, std of speed, count
lon_index= findgen(nbin)/nbin*360. - 180.
lat_index= findgen(nbin/2)/nbin*360. - 90.

; Create processing arrays
p_edgexyz= fltarr(np, 3) ; xyz of each particle starting position on the crater edge
normv= fltarr(np, 3)  ; xyz vector normal to the surface for each particle
centv= fltarr(3) ; xyz vector from center of crater to edge (starting point, each particle)
s_v= dblarr(6)  ; State vector of particle: xyz, 3-vector velocity

; Get locations of crater edge for each particle
crater_edge, [cr,cr,cr], r_crater, cll, np, p_edge
; Convert lat,lon to xyz for each point
for i_p= 0, np-1 do begin
  p_e_in= p_edge[i_p, *]
  llsurf_to_xyz, p_e_in, dtmllr, llr, p_e_out
  p_edgexyz[i_p,*]= p_e_out
endfor

; Using radial vector for surface normal
normv= p_edgexyz

; If debugging, stop after setup
if dbug then stop,'Done with setup'

; For each location on the crater edge
for inp= 0, np-1 do begin
  ; Get vector from center of crater to edge point
  centv= p_edgexyz[inp,*]-cllxyz[*]
  ; Get unit vector perpendicular to center vector and surface normal
  k0= crossp(normv[inp,*], centv)
  k= k0/sqrt(k0[0]^2.+k0[1]^2.+k0[2]^2.)
  ; Extract normal vector for this particle from array with all particles
  nv= normv[inp,*]
  ; Rotate normal vector by vang to get initial velocity vector of ejected particle
  kvdot= k[0]*nv[0]+ k[1]*nv[1]+ k[2]*nv[2]
  v0= nv*cos(vang)+ crossp(k,nv)*sin(vang)+ k*kvdot*(1.0-cos(vang))
  
  ; Normalize this velocity vector
  v0_hat= v0 / sqrt(v0[0]^2.+v0[1]^2.+v0[2]^2.)
  
  ; For each velocity
  for iv= 0, nxadj-1 do begin  ; Skip nx+1 point as it is zero velocity
    ; Insert initial location into state vector
    s_v[0:2]= p_edgexyz[inp,*]
    ; Insert velocity into state vector
    s_v[3:5]= v_x[iv] * v0_hat
    ; Add velocity due to rotation
    s_v[3:5]= s_v[3:5] + [-omega*s_v[1],omega*s_v[0], 0.]
    ; Launch particle (one time step)
    cspice_prop2b, GM, s_v, t_delta, s_v_out
    s_v= s_v_out
    ; Calculate altitude
    xyz_to_llr, s_v[0:2], omega, t_delta, ll, r ;Convert xyz to lan, lat, and radius
    llsurf_to_xyz, ll, dtmllr, surf_r, discard ; Surface radius at this lon and lat
    p_alt= r - surf_r
    ; Initialize time counter
    t_tot= t_delta  ; Already did first time step
    ; Do until particle hits the surface
    while ((p_alt ge 0.0) and (t_tot le t_max))do begin
      ; Propagate for t_delta
      cspice_prop2b, GM, s_v, t_delta, s_v_out
      s_v= s_v_out
      t_tot=  t_tot + t_delta ;Increment total time
      ; Calculate new altitude
      xyz_to_llr, s_v[0:2], omega, t_tot, ll, r ;Convert xyz to lan, lat, and radius
      llsurf_to_xyz, ll, dtmllr, surf_r, discard ; Surface radius at this lon and lat
      p_alt= r - surf_r
    endwhile  ; End if above surface
    
    ; Store  output: p_out[inp, iv, 6]= 0=lat,1=lon,2=angle,3=v,4=v_surface,5=time
    r= sqrt( s_v[0]^2 + s_v[1]^2 + s_v[2]^2 )
    p_out[inp, iv, 0]= ll[0] ; Longitude
    p_out[inp, iv, 1]= ll[1] ; Latitude
    ; Store velocity at impact
    v_imp= s_v[3:5] - [-omega*s_v[1],omega*s_v[0], 0.]
    v_imp_mag= sqrt(v_imp[0]^2.+v_imp[1]^2.+v_imp[2]^2.)  ; Magnitude of velocity at impact
    p_out[inp,iv,3]= v_imp_mag  ; Impact velocity
    ; Calculate and store impact angle
    s_norm= -s_v[0:2]  ;Surface normal (negative)
    s_norm_mag= sqrt(s_norm[0]^2. + s_norm[1]^2. + s_norm[2]^2.)  ;Magnitude of surface normal
    snimdot= s_norm[0]*v_imp[0]+ s_norm[1]*v_imp[1]+ s_norm[2]*v_imp[2] ; Dot product of surface normal and velocity
    p_out[inp,iv,2]= 90. - !radeg * acos(snimdot/s_norm_mag/v_imp_mag) ; Impact angle
    ; Calculate and store velocity along surface
    p_out[inp,iv,4]= p_out[inp,iv,3]*cos(p_out[inp,iv,2]/!radeg) ;Surface velocity
    ; Store time to impact
    p_out[inp,iv,5]= t_tot ; Time at impact

    ; Add this particle to the surface grid
    i_lon_arr= where(lon_index gt ll[0])
    i_lat_arr= where(lat_index gt ll[1])
    i_lon= i_lon_arr[0]-1
    i_lat= i_lat_arr[0]-1
    s_out[i_lon, i_lat, 0]= s_out[i_lon, i_lat, 0] + dm[iv] ; Total mass in this lat/lon bin
    s_out[i_lon, i_lat, 1]= s_out[i_lon, i_lat, 1] + p_out[inp,iv,4] ; Velocity along surface
    s_out[i_lon, i_lat, 2]= s_out[i_lon, i_lat, 2] + p_out[inp,iv,4]^2. ; Squared velocity along surface
    s_out[i_lon, i_lat, 3]= s_out[i_lon, i_lat, 3] + 1.  ; Count of particles 

    if dbug then stop,'Stop for location, ix',inp, iv
    
  endfor  ; End for each velocity
  
    ; Print status message
    if (inp mod (np/5)) eq 0 then print, 'Done location', inp
  
endfor  ; End for each location on crater edge

; Calculate the average and standard deviation for each lon/lat bin
for illon= 0,  nbin-1 do begin
  for illat= 0, nbin/2-1 do  begin
    if s_out[illon, illat, 3] ne 0.0 then begin ; Check if any data in this bin
      s_out[illon, illat, 1]= s_out[illon, illat, 1] / s_out[illon, illat, 3]
      s_out[illon, illat, 2]= sqrt(s_out[illon, illat, 1]/s_out[illon, illat, 3] - s_out[illon, illat, 1]^2.)
    endif
  endfor
endfor

stop, 'Stopping for checking and plotting'

return
end
